@@ -1,3 +1,9 @@
+use std::convert::TryInto;
+use std::env;
+use std::io::{Read, Seek, SeekFrom, Write};
+use std::path::{Path, PathBuf};
+use std::process;
+
 use anyhow::{anyhow, Result};
 use atty::Stream;
 use clap::ArgMatches;
@@ -7,12 +13,6 @@ use dialoguer::Confirm;
 use nix::unistd::{fork, ForkResult};
 use secrecy::{ExposeSecret, Secret};
 
-use std::convert::TryInto;
-use std::env;
-use std::io::{Read, Seek, SeekFrom, Write};
-use std::path::{Path, PathBuf};
-use std::process;
-
 use crate::kbs2::agent;
 use crate::kbs2::backend::{self, Backend};
 use crate::kbs2::config::{self, Pinentry};
@@ -64,8 +64,9 @@ pub fn agent(matches: &ArgMatches, config: &config::Config) -> Result<()> {
 
     // No subcommand: run the agent itself
     match matches.subcommand() {
-        Some(("flush", matches)) => agent_flush(&matches),
-        Some(("unwrap", matches)) => agent_unwrap(&matches, &config),
+        Some(("flush", matches)) => agent_flush(matches),
+        Some(("query", matches)) => agent_query(matches, config),
+        Some(("unwrap", matches)) => agent_unwrap(matches, config),
         _ => unreachable!(),
     }
 }
@@ -84,6 +85,27 @@ fn agent_flush(matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
+/// Implements the `kbs2 agent query` subcommand.
+fn agent_query(_matches: &ArgMatches, config: &config::Config) -> Result<()> {
+    log::debug!("querying the agent for a key's existence");
+
+    // It doesn't make sense to query the agent for keypairs that the agent
+    // doesn't manage. Use a specific code to signal this case.
+    if !config.wrapped {
+        std::process::exit(2);
+    }
+
+    // Don't allow client creation to fail the normal way: if we can't create
+    // a client for whatever reason (e.g., the agent isn't running), exit
+    // with a specific code to signal our state to the user.
+    let client = agent::Client::new().unwrap_or_else(|_| std::process::exit(3));
+    if !client.query_key(&config.public_key)? {
+        std::process::exit(1);
+    }
+
+    Ok(())
+}
+
 /// Implements the `kbs2 agent unwrap` subcommand.
 fn agent_unwrap(_matches: &ArgMatches, config: &config::Config) -> Result<()> {
     log::debug!("asking the agent to unwrap a key");
@@ -94,13 +116,13 @@ fn agent_unwrap(_matches: &ArgMatches, config: &config::Config) -> Result<()> {
     }
 
     let client = agent::Client::new()?;
-    if client.query_key(&config.keyfile)? {
+    if client.query_key(&config.public_key)? {
         println!("kbs2 agent already has this key; ignoring.");
         return Ok(());
     }
 
     let password = util::get_password(None, &config.pinentry)?;
-    client.add_key(&config.keyfile, password)?;
+    client.add_key(&config.public_key, &config.keyfile, password)?;
 
     Ok(())
 }
@@ -149,7 +171,7 @@ pub fn new(matches: &ArgMatches, config: &config::Config) -> Result<()> {
 
     if let Some(post_hook) = &session.config.commands.new.post_hook {
         log::debug!("post-hook: {}", post_hook);
-        session.config.call_hook(post_hook, &[&label])?;
+        session.config.call_hook(post_hook, &[label])?;
     }
 
     Ok(())
@@ -165,7 +187,7 @@ fn new_login(
     let fields = input::fields(
         &[Insensitive("Username"), Sensitive("Password")],
         terse,
-        &session.config,
+        session.config,
         generator,
     )?;
     let record = record::Record::login(label, &fields[0], &fields[1]);
@@ -183,7 +205,7 @@ fn new_environment(
     let fields = input::fields(
         &[Insensitive("Variable"), Sensitive("Value")],
         terse,
-        &session.config,
+        session.config,
         generator,
     )?;
     let record = record::Record::environment(label, &fields[0], &fields[1]);
@@ -198,12 +220,7 @@ fn new_unstructured(
     session: &Session,
     generator: Option<&dyn Generator>,
 ) -> Result<()> {
-    let fields = input::fields(
-        &[Insensitive("Contents")],
-        terse,
-        &session.config,
-        generator,
-    )?;
+    let fields = input::fields(&[Insensitive("Contents")], terse, session.config, generator)?;
     let record = record::Record::unstructured(label, &fields[0]);
 
     session.add_record(&record)
@@ -253,12 +270,15 @@ pub fn rm(matches: &ArgMatches, config: &config::Config) -> Result<()> {
     let session: Session = config.try_into()?;
 
     #[allow(clippy::unwrap_used)]
-    let label = matches.value_of("label").unwrap();
-    session.delete_record(label)?;
+    let labels: Vec<_> = matches.values_of("label").unwrap().collect();
+
+    for label in &labels {
+        session.delete_record(label)?;
+    }
 
     if let Some(post_hook) = &session.config.commands.rm.post_hook {
         log::debug!("post-hook: {}", post_hook);
-        session.config.call_hook(post_hook, &[&label])?;
+        session.config.call_hook(post_hook, &labels)?;
     }
 
     Ok(())
@@ -271,22 +291,25 @@ pub fn dump(matches: &ArgMatches, config: &config::Config) -> Result<()> {
     let session: Session = config.try_into()?;
 
     #[allow(clippy::unwrap_used)]
-    let label = matches.value_of("label").unwrap();
-    let record = session.get_record(&label)?;
+    let labels: Vec<_> = matches.values_of("label").unwrap().collect();
 
-    if matches.is_present("json") {
-        println!("{}", serde_json::to_string(&record)?);
-    } else {
-        println!("Label {}\nKind {}", label, record.body);
+    for label in labels {
+        let record = session.get_record(label)?;
 
-        match record.body {
-            RecordBody::Login(l) => {
-                println!("Username {}\nPassword {}", l.username, l.password)
+        if matches.is_present("json") {
+            println!("{}", serde_json::to_string(&record)?);
+        } else {
+            println!("Label {}\nKind {}", label, record.body);
+
+            match record.body {
+                RecordBody::Login(l) => {
+                    println!("Username {}\nPassword {}", l.username, l.password)
+                }
+                RecordBody::Environment(e) => {
+                    println!("Variable {}\nValue {}", e.variable, e.value)
+                }
+                RecordBody::Unstructured(u) => println!("Contents {}", u.contents),
             }
-            RecordBody::Environment(e) => {
-                println!("Variable {}\nValue {}", e.variable, e.value)
-            }
-            RecordBody::Unstructured(u) => println!("Contents {}", u.contents),
         }
     }
 
@@ -306,7 +329,7 @@ pub fn pass(matches: &ArgMatches, config: &config::Config) -> Result<()> {
 
     #[allow(clippy::unwrap_used)]
     let label = matches.value_of("label").unwrap();
-    let record = session.get_record(&label)?;
+    let record = session.get_record(label)?;
 
     let login = match record.body {
         RecordBody::Login(l) => l,
@@ -422,7 +445,7 @@ pub fn env(matches: &ArgMatches, config: &config::Config) -> Result<()> {
 
     #[allow(clippy::unwrap_used)]
     let label = matches.value_of("label").unwrap();
-    let record = session.get_record(&label)?;
+    let record = session.get_record(label)?;
 
     let environment = match record.body {
         RecordBody::Environment(e) => e,
@@ -465,7 +488,7 @@ pub fn edit(matches: &ArgMatches, config: &config::Config) -> Result<()> {
 
     #[allow(clippy::unwrap_used)]
     let label = matches.value_of("label").unwrap();
-    let record = session.get_record(&label)?;
+    let record = session.get_record(label)?;
 
     let mut file = tempfile::NamedTempFile::new()?;
     file.write_all(&serde_json::to_vec_pretty(&record)?)?;
@@ -628,7 +651,7 @@ pub fn rekey(matches: &ArgMatches, config: &config::Config) -> Result<()> {
         let records: Result<Vec<record::Record>> = session
             .record_labels()?
             .iter()
-            .map(|l| session.get_record(&l))
+            .map(|l| session.get_record(l))
             .collect();
 
         records?.into_iter().map(Secret::new).collect()
@@ -660,7 +683,7 @@ pub fn rekey(matches: &ArgMatches, config: &config::Config) -> Result<()> {
     {
         let client = agent::Client::new()?;
         client.flush_keys()?;
-        client.add_key(&config.keyfile, new_password)?;
+        client.add_key(&config.public_key, &config.keyfile, new_password)?;
     }
 
     // Create a new session from the new config and use it to re-encrypt each record.
