@@ -20,7 +20,10 @@ pub static CONFIG_BASEDIR: &str = "kbs2";
 
 /// The default basename for the main config file, relative to the configuration
 /// directory.
-pub static CONFIG_BASENAME: &str = "kbs2.conf";
+pub static CONFIG_BASENAME: &str = "config.toml";
+
+/// A deprecated alternative default config basename.
+pub static LEGACY_CONFIG_BASENAME: &str = "kbs2.conf";
 
 /// The default generate age key is placed in this file, relative to
 /// the configuration directory.
@@ -177,8 +180,9 @@ impl AsRef<OsStr> for Pinentry {
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[serde(untagged)]
 pub enum GeneratorConfig {
-    Command(GeneratorCommandConfig),
-    Internal(GeneratorInternalConfig),
+    Command(ExternalGeneratorConfig),
+    Internal(InternalGeneratorConfig),
+    InternalLegacy(LegacyInternalGeneratorConfig),
 }
 
 impl GeneratorConfig {
@@ -186,13 +190,14 @@ impl GeneratorConfig {
         match self {
             GeneratorConfig::Command(g) => g as &dyn Generator,
             GeneratorConfig::Internal(g) => g as &dyn Generator,
+            GeneratorConfig::InternalLegacy(g) => g as &dyn Generator,
         }
     }
 }
 
-/// The configuration settings for a "command" generator.
+/// The configuration settings for an external (i.e., separate command) generator.
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct GeneratorCommandConfig {
+pub struct ExternalGeneratorConfig {
     /// The name of the generator.
     pub name: String,
 
@@ -200,31 +205,50 @@ pub struct GeneratorCommandConfig {
     pub command: String,
 }
 
-/// The configuration settings for an "internal" generator.
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct GeneratorInternalConfig {
+pub struct InternalGeneratorConfig {
+    /// The name of the generator.
+    pub name: String,
+
+    /// The alphabets used by the generator.
+    pub alphabets: Vec<String>,
+
+    /// The length of the secrets generated.
+    pub length: usize,
+}
+
+impl Default for InternalGeneratorConfig {
+    fn default() -> Self {
+        InternalGeneratorConfig {
+            name: "default".into(),
+            alphabets: vec![
+                "abcdefghijklmnopqrstuvwxyz".into(),
+                "ABCDEFGHIJKLMNOPQRSTUVWXYZ".into(),
+                "0123456789".into(),
+                "(){}[]-_+=".into(),
+            ],
+            length: 16,
+        }
+    }
+}
+
+/// The configuration settings for a legacy "internal" generator.
+///
+/// This is a **legacy** generator that will be removed in an upcoming release.
+///
+/// Users should prefer `InternalGeneratorConfig`.
+#[derive(Clone, Debug, Deserialize, Serialize)]
+pub struct LegacyInternalGeneratorConfig {
     /// The name of the generator.
     pub name: String,
 
     /// The alphabet to sample from when generating a secret.
     pub alphabet: String,
 
-    /// The number of characters to sample from the alphabet.
+    /// The length of the secrets generated.
     pub length: u32,
 }
 
-impl Default for GeneratorInternalConfig {
-    fn default() -> Self {
-        GeneratorInternalConfig {
-            name: "default".into(),
-            // NOTE(ww): This alphabet should be a decent default, as it contains
-            // symbols but not commonly blacklisted ones (e.g. %, $).
-            alphabet: "abcdefghijklmnopqrstuvwxyz0123456789(){}[]-_+=".into(),
-            length: 16,
-        }
-    }
-}
-
 /// The per-command configuration settings known to `kbs2`.
 #[derive(Clone, Default, Debug, Deserialize, Serialize)]
 #[serde(default)]
@@ -424,15 +448,36 @@ pub fn initialize<P: AsRef<Path>>(
 pub fn load<P: AsRef<Path>>(config_dir: P) -> Result<Config> {
     let config_dir = config_dir.as_ref();
     let config_path = config_dir.join(CONFIG_BASENAME);
-    let contents = fs::read_to_string(config_path)?;
 
-    Ok(Config {
+    let contents = if config_path.is_file() {
+        fs::read_to_string(config_path)?
+    } else {
+        // Try the legacy config file. This behavior will be removed in a future stable release.
+        util::warn(&format!(
+            "{} not found in config dir; trying {}",
+            CONFIG_BASENAME, LEGACY_CONFIG_BASENAME
+        ));
+        util::warn("note: this behavior will be removed in a future stable release");
+        fs::read_to_string(config_dir.join(LEGACY_CONFIG_BASENAME))?
+    };
+
+    let config = Config {
         config_dir: config_dir
             .to_str()
             .ok_or_else(|| anyhow!("unrepresentable config dir path: {:?}", config_dir))?
             .into(),
         ..toml::from_str(&contents).map_err(|e| anyhow!("config loading error: {}", e))?
-    })
+    };
+
+    // Warn if the user has any old-style generators.
+    for gen in config.generators.iter() {
+        if matches!(gen, GeneratorConfig::InternalLegacy(_)) {
+            util::warn(&format!("loaded legacy generator: {}", gen.as_dyn().name()));
+            util::warn("note: this behavior will be removed in a future stable release");
+        }
+    }
+
+    Ok(config)
 }
 
 #[cfg(test)]
